package com.tencent.tdmq.rabbitmq.demo.deadletter;

import com.rabbitmq.client.AMQP;
import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import com.rabbitmq.client.DefaultConsumer;
import com.rabbitmq.client.Envelope;
import com.tencent.tdmq.rabbitmq.demo.Constant;

import java.io.IOException;
import java.nio.charset.StandardCharsets;

/**
 * 死信队列消费者
 */
public class HandleDeadLetter {
    /**
     * 消息队列名称
     */
    public static final String QUEUE_NAME = "dead_letter_queue";

    public static void main(String[] args) throws Exception {
        // 连接工厂
        ConnectionFactory factory = new ConnectionFactory();
        // 设置服务地址
        factory.setUri(Constant.URI);
        // 设置Virtual Hosts
        factory.setVirtualHost(Constant.VHOST_NAME);
        // 设置用户名
        factory.setUsername(Constant.USERNAME);
        // 设置密码
        factory.setPassword(Constant.PASSWORD);
        // 获取连接、建立通道
        Connection connection = factory.newConnection();
        Channel channel = connection.createChannel();
        channel.exchangeDeclare("my-deadLetter-exchange", "fanout");
        // 绑消息队列到交换机
        channel.queueBind(QUEUE_NAME, "my-deadLetter-exchange", "");
        System.out.println(" [Consumer1] Waiting for messages. To exit press CTRL+C");
        // 订阅消息 (设置消息确认为false，由消费者进行手动ack)
        channel.basicConsume(QUEUE_NAME, false, "ConsumerTag", new DefaultConsumer(channel) {
            @Override
            public void handleDelivery(String consumerTag, Envelope envelope,
                                       AMQP.BasicProperties properties, byte[] body) throws IOException {
                try {
                    // 接收到的消息，进行业务逻辑处理。
                    System.out.println("Received： " + new String(body, StandardCharsets.UTF_8) + ", deliveryTag: " + envelope.getDeliveryTag() + ", messageId: " + properties.getMessageId());
                    // 确认消息
                    channel.basicAck(envelope.getDeliveryTag(), false);
                } catch (Exception e) {
                    // 处理失败回复 (第三个参数为true表示消息将回到消息队列)
                    channel.basicNack(envelope.getDeliveryTag(), false, true);
                    e.printStackTrace();
                }
            }
        });
    }
}
