package com.tencent.tdmq.rabbitmq.demo.deadletter;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import com.rabbitmq.client.DeliverCallback;
import com.tencent.tdmq.rabbitmq.demo.Constant;

import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;

/**
 * 产生死信的原因
 * 1、消费者拒绝且requeue参数为false，或nack处理时requeue参数为false
 * 2、消息过期
 * 3、由于消息队列已满，消息被丢弃
 */
public class MessageConsumer {


    /**
     * 消费者监听消息队列名称
     */
    public static final String QUEUE_NAME = "my_normal_queue";


    public static void main(String[] args) throws Exception {
        // 连接工厂
        ConnectionFactory factory = new ConnectionFactory();
        // 设置服务地址
        factory.setUri(Constant.URI);
        // 设置Virtual Hosts
        factory.setVirtualHost(Constant.VHOST_NAME);
        // 设置用户名
        factory.setUsername(Constant.USERNAME);
        // 设置密码
        factory.setPassword(Constant.PASSWORD);
        // 获取连接、建立通道
        try (Connection connection = factory.newConnection(); Channel channel = connection.createChannel()) {
            Map<String, Object> requestParam = new HashMap<>();
            requestParam.put("x-dead-letter-exchange", "my-deadLetter-exchange");
            requestParam.put("x-message-ttl", 60000);
            // 绑定消息队列
            channel.queueDeclare(QUEUE_NAME, true, false, false, requestParam);
            System.out.println(" [*] Waiting for messages. To exit press CTRL+C");
            // 消息处理回调；当收到消息后会执行该消息处理逻辑
            DeliverCallback deliverCallback = (consumerTag, delivery) -> {
                String message = new String(delivery.getBody(), StandardCharsets.UTF_8);
                System.out.println(" [x] Received '" + message + "'");
                // 消息重试  第三个参数设置为true
                channel.basicNack(delivery.getEnvelope().getDeliveryTag(), false, false);
///                // 消息重试  第二个参数设置为true
///                channel.basicReject(delivery.getEnvelope().getDeliveryTag(), true);
            };
            // 绑定消费者与消息队列
            channel.basicConsume(QUEUE_NAME, false, deliverCallback, consumerTag -> {
                System.out.println("consumerTag = " + consumerTag);
            });

            Thread.sleep(100_000);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
