package com.tencent.tdmq.rabbitmq.demo.helloworld;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import com.tencent.tdmq.rabbitmq.demo.Constant;

/**
 * rabbitmq Hello World 模型示例
 * 消息生产者
 */
public class MessageProducer {

    /**
     * 消息队列名称
     */
    public static final String QUEUE_NAME = "hello-world-queue";


    public static void main(String[] args) throws Exception {
        // 连接工厂
        ConnectionFactory factory = new ConnectionFactory();
        // 设置服务地址
        factory.setUri(Constant.URI);
        // 设置Virtual Hosts
        factory.setVirtualHost(Constant.VHOST_NAME);
        // 设置用户名
        factory.setUsername(Constant.USERNAME);
        // 设置密码
        factory.setPassword(Constant.PASSWORD);
        Connection connection = null;
        Channel channel = null;
        try {
            // 获取连接
            connection = factory.newConnection();
            // 建立通道
            channel = connection.createChannel();
            // 绑定消息队列
            channel.queueDeclare(QUEUE_NAME, true, false, false, null);
            String message = "Hello World!";
            // 发布消息 (hello world 消息模型无需指定交换机类型)
            channel.basicPublish("", QUEUE_NAME, null, message.getBytes());
            System.out.println(" [Producer(hello world)] Sent '" + message + "'");
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            // 释放资源
            if (channel != null) {
                channel.close();
            }
            if (connection != null) {
                connection.close();
            }
        }
    }
}
