package com.tencent.tdmq.rabbitmq.demo.publishandsubscribe;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import com.rabbitmq.client.DeliverCallback;
import com.tencent.tdmq.rabbitmq.demo.Constant;

import java.nio.charset.StandardCharsets;

/**
 * fanout 模型示例
 * 消息消费者
 */
public class MessageConsumer2 {

    public static final String QUEUE_NAME = "ps_queue1";

    /**
     * 交换机名称
     */
    private static final String EXCHANGE_NAME = "logs";

    public static void main(String[] args) throws Exception {
        // 连接工厂
        ConnectionFactory factory = new ConnectionFactory();
        // 设置服务地址
        factory.setUri(Constant.URI);
        // 设置Virtual Hosts
        factory.setVirtualHost(Constant.VHOST_NAME);
        // 设置用户名
        factory.setUsername(Constant.USERNAME);
        // 设置密码
        factory.setPassword(Constant.PASSWORD);
        // 获取连接、建立通道
        try (Connection connection = factory.newConnection(); Channel channel = connection.createChannel()) {
            // 绑定消息交换机
            channel.exchangeDeclare(EXCHANGE_NAME, "fanout");
            // 获取随机消息队列名称
            channel.queueDeclare(QUEUE_NAME, true, false, false, null);
            // 绑消息队列到交换机
            channel.queueBind(QUEUE_NAME, EXCHANGE_NAME, "");
            System.out.println(" [Consumer2(pub/sub)] Waiting for messages.");
            DeliverCallback deliverCallback = (consumerTag, delivery) -> {
                String message = new String(delivery.getBody(), StandardCharsets.UTF_8);
                System.out.println("Received '" + message + "'");
            };
            channel.basicConsume(QUEUE_NAME, true, deliverCallback, consumerTag -> {
            });
            Thread.sleep(100_000);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
