package com.tencent.tdmq.rabbitmq.demo.publishandsubscribe;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import com.tencent.tdmq.rabbitmq.demo.Constant;

/**
 * fanout 模型示例
 * 消息生产者
 */
public class MessageProducer {

    /**
     * 交换机名称
     */
    private static final String EXCHANGE_NAME = "logs";

    public static void main(String[] args) throws Exception {
        // 连接工厂
        ConnectionFactory factory = new ConnectionFactory();
        // 设置服务地址
        factory.setUri(Constant.URI);
        // 设置Virtual Hosts
        factory.setVirtualHost(Constant.VHOST_NAME);
        // 设置用户名
        factory.setUsername(Constant.USERNAME);
        // 设置密码
        factory.setPassword(Constant.PASSWORD);
        // 获取连接、建立通道
        try (Connection connection = factory.newConnection(); Channel channel = connection.createChannel()) {
            // 绑定消息交换机
            channel.exchangeDeclare(EXCHANGE_NAME, "fanout");
            for (int i = 0; i < 10; i++) {
                String message = "this is fanout message " + i;
                // 发布消息到指定队列并设置消息持久化 （无需指定交换机）
                channel.basicPublish(EXCHANGE_NAME, "", null, message.getBytes());
                System.out.println(" [Producer(pub/sub) Sent '" + message + "'");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
