package com.tencent.tdmq.rabbitmq.demo.publisherconfirms;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import com.rabbitmq.client.MessageProperties;
import com.tencent.tdmq.rabbitmq.demo.Constant;

import java.io.IOException;
import java.net.URISyntaxException;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.util.concurrent.TimeoutException;

/**
 * 确认消息成功到达服务端，除了生产者确认机制实现外，还可以使用事务机制
 * 当事务成功提交也能确保消息发送成功
 */
public class PublisherTx {

    public static void main(String[] args) throws IOException, TimeoutException, URISyntaxException, NoSuchAlgorithmException, KeyManagementException {
        // 连接工厂
        ConnectionFactory factory = new ConnectionFactory();
        // 设置服务地址
        factory.setUri(Constant.URI);
        // 设置Virtual Hosts
        factory.setVirtualHost(Constant.VHOST_NAME);
        // 设置用户名
        factory.setUsername(Constant.USERNAME);
        // 设置密码
        factory.setPassword(Constant.PASSWORD);
        // 获取连接
        Connection connection = factory.newConnection();
        // 创建通道
        Channel channel = connection.createChannel();
        // 开启事务
        channel.txSelect();
        // 绑定消息交换机
        channel.exchangeDeclare("tx-exchange", "fanout");
        String message = "this is tx-exchange message.";
        try {
            // 发布消息到指定队列并设置消息持久化 （无需指定交换机）
            channel.basicPublish("tx-exchange", "", MessageProperties.PERSISTENT_TEXT_PLAIN, message.getBytes());
            // 提交事务
            channel.txCommit();
        } catch (IOException e) {
            // 回滚事务
            channel.txRollback();
            e.printStackTrace();
        }
    }
}
