package com.tencent.tdmq.rabbitmq.demo.topics;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import com.rabbitmq.client.DeliverCallback;
import com.tencent.tdmq.rabbitmq.demo.Constant;

import java.nio.charset.StandardCharsets;

/**
 * topics 模型示例
 * 消息消费者
 */
public class MessageConsumer3 {

    /**
     * 消息队列名称
     */
    public static final String QUEUE_NAME = "topic_queue3";

    /**
     * 交换机名称
     */
    private static final String EXCHANGE_NAME = "topic_demo";

    public static void main(String[] args) throws Exception {
        // 连接工厂
        ConnectionFactory factory = new ConnectionFactory();
        // 设置服务地址
        factory.setUri(Constant.URI);
        // 设置Virtual Hosts
        factory.setVirtualHost(Constant.VHOST_NAME);
        // 设置用户名
        factory.setUsername(Constant.USERNAME);
        // 设置密码
        factory.setPassword(Constant.PASSWORD);
        // 获取连接、建立通道
        try (Connection connection = factory.newConnection(); Channel channel = connection.createChannel()) {
            // 绑定消息交换机
            channel.exchangeDeclare(EXCHANGE_NAME, "topic");
            // 绑消息队列到交换机
            channel.queueBind(QUEUE_NAME, EXCHANGE_NAME, "*.*.zzz");
            System.out.println(" [Consumer3[*.*.ccc] Waiting for messages.");
            DeliverCallback deliverCallback = (consumerTag, delivery) -> {
                try {
                    // 接收到的消息，进行业务逻辑处理。
                    System.out.println("Received： " + new String(delivery.getBody(), StandardCharsets.UTF_8) + ", deliveryTag: " + delivery.getEnvelope().getDeliveryTag() + ", messageId: " + delivery.getProperties().getMessageId());
                    // 确认消息
                    channel.basicAck(delivery.getEnvelope().getDeliveryTag(), false);
                } catch (Exception e) {
                    // 处理失败回复 (第二个参数为true表示消息将回到消息队列)
                    channel.basicNack(delivery.getEnvelope().getDeliveryTag(), false, true);
                    e.printStackTrace();
                }
            };
            // 订阅消息 (设置消息确认为false，由消费者进行手动ack)
            channel.basicConsume(QUEUE_NAME, false, deliverCallback, consumerTag -> {
            });
            Thread.sleep(100_000);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
