package com.tencent.tdmq.rabbitmq.demo.topics;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import com.tencent.tdmq.rabbitmq.demo.Constant;

/**
 * Topics 模型示例
 * 消息生产者
 */
public class MessageProducer {

    /**
     * 指定交换机名称
     */
    public static final String EXCHANGE_NAME = "topic_demo";

    /**
     * 路由key
     */
    public static final String[] ROUTING_KEYS = new String[]{"aaa.bbb.ccc", "ccc.bbb.aaa", "aaa.aaa.aaa", "aaa.yyy.zzz", "xxx.yyy.zzz"};

    public static void main(String[] args) throws Exception {
        // 连接工厂
        ConnectionFactory factory = new ConnectionFactory();
        // 设置服务地址
        factory.setUri(Constant.URI);
        // 设置Virtual Hosts
        factory.setVirtualHost(Constant.VHOST_NAME);
        // 设置用户名
        factory.setUsername(Constant.USERNAME);
        // 设置密码
        factory.setPassword(Constant.PASSWORD);
        // 获取连接、建立通道
        try (Connection connection = factory.newConnection(); Channel channel = connection.createChannel()) {
            // 绑定消息交换机
            channel.exchangeDeclare(EXCHANGE_NAME, "topic");
            for (String routingKey : ROUTING_KEYS) {
                String message = "this is topic message. message type [" + routingKey + "]";
                // 发布消息 （无需指定交换机）
                channel.basicPublish(EXCHANGE_NAME, routingKey, null, message.getBytes());
                System.out.println(" [producer(topic)] Sent '" + message + "'");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
