package com.tencent.tdmq.rabbitmq.demo.workqueue;

import com.rabbitmq.client.AMQP;
import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import com.rabbitmq.client.DefaultConsumer;
import com.rabbitmq.client.Envelope;
import com.tencent.tdmq.rabbitmq.demo.Constant;

import java.io.IOException;
import java.nio.charset.StandardCharsets;

/**
 * work queues 模型示例
 * 消息消费者1
 */
public class MessageConsumer1 {

    /**
     * 消息队列名称
     */
    public static final String QUEUE_NAME = "work_queue";

    public static void main(String[] args) throws Exception {
        // 连接工厂
        ConnectionFactory factory = new ConnectionFactory();
        // 设置服务地址
        factory.setUri(Constant.URI);
        // 设置Virtual Hosts
        factory.setVirtualHost(Constant.VHOST_NAME);
        // 设置用户名
        factory.setUsername(Constant.USERNAME);
        // 设置密码
        factory.setPassword(Constant.PASSWORD);
        // 获取连接、建立通道
        Connection connection = factory.newConnection();
        Channel channel = connection.createChannel();
        // 消息投递限制
        channel.basicQos(1);
        // 绑定消息队列
        channel.queueDeclare(QUEUE_NAME, true, false, false, null);
        System.out.println("[Work1] Waiting for messages.");
        // 订阅消息 (设置消息确认为false，由消费者进行手动ack)
        channel.basicConsume(QUEUE_NAME, false, "ConsumerTag", new DefaultConsumer(channel) {
            @Override
            public void handleDelivery(String consumerTag, Envelope envelope,
                                       AMQP.BasicProperties properties, byte[] body) throws IOException {
                // 接收到的消息，进行业务逻辑处理。
                System.out.println("Received： " + new String(body, StandardCharsets.UTF_8) + ", deliveryTag: " + envelope.getDeliveryTag() + ", messageId: " + properties.getMessageId());
                try {
                    // 模拟业务处理
                    doWork(new String(body, StandardCharsets.UTF_8));
                    // 确认消息
                    channel.basicAck(envelope.getDeliveryTag(), false);
                } catch (Exception e) {
                    // 处理失败回复 (第三个参数为true表示消息将回到消息队列)
                    channel.basicNack(envelope.getDeliveryTag(), false, true);
                    e.printStackTrace();
                }
            }
        });
    }

    private static void doWork(String task) throws InterruptedException {
        Thread.sleep(1000);
    }
}
