package com.tencent.tdmq.rabbitmq.demo.workqueue;

import com.rabbitmq.client.Channel;
import com.rabbitmq.client.Connection;
import com.rabbitmq.client.ConnectionFactory;
import com.rabbitmq.client.DeliverCallback;
import com.tencent.tdmq.rabbitmq.demo.Constant;

import java.nio.charset.StandardCharsets;

/**
 * work queues 模型示例
 * 消息消费者2
 */
public class MessageConsumer2 {

    /**
     * 消息队列名称
     */
    public static final String QUEUE_NAME = "work_queue";

    public static void main(String[] args) throws Exception {
        // 连接工厂
        ConnectionFactory factory = new ConnectionFactory();
        // 设置服务地址
        factory.setUri(Constant.URI);
        // 设置Virtual Hosts
        factory.setVirtualHost(Constant.VHOST_NAME);
        // 设置用户名
        factory.setUsername(Constant.USERNAME);
        // 设置密码
        factory.setPassword(Constant.PASSWORD);
        // 获取连接、建立通道
        try (Connection connection = factory.newConnection(); Channel channel = connection.createChannel()) {
            channel.basicQos(1);
            // 绑定消息队列
            channel.queueDeclare(QUEUE_NAME, true, false, false, null);
            System.out.println(" [Work2] Waiting for messages.");
            // 消息处理回调；当收到消息后会执行该消息处理逻辑
            DeliverCallback deliverCallback = (consumerTag, delivery) -> {
                String message = new String(delivery.getBody(), StandardCharsets.UTF_8);
                System.out.println(" [work2] Received '" + message + "'");
                try {
                    // 模拟业务处理
                    doWork(message);
                    // 确认消息
                    channel.basicAck(delivery.getEnvelope().getDeliveryTag(), false);
                } catch (Exception e) {
                    // 处理失败将消息返回队列
                    channel.basicNack(delivery.getEnvelope().getDeliveryTag(), false, true);
                    e.printStackTrace();
                }
            };
            // 订阅消息 (设置消息确认为false，由消费者进行手动ack)
            channel.basicConsume(QUEUE_NAME, false, deliverCallback, consumerTag -> {
                System.out.println("consumerTag = " + consumerTag);
            });
            Thread.sleep(100_000);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private static void doWork(String task) throws InterruptedException {
        Thread.sleep(5000);
    }
}
